import re
import time
import unicodedata
import ruian  # https://service.sysnet.cz/SYSNET/RUIAN/1.0.2/ui/
from itertools import chain

from elasticsearch import ApiError
from pynominatim import Nominatim, NominatimReverse
# from data_mining.elk import ELASTICSEARCH_CLIENT
# import config
# from data_mining.get_tweepy import get_tweets

from shopping.config import RUIAN_API, RUIAN_HOST
from shopping.elk import ELASTICSEARCH_CLIENT
from shopping.get_tweepy import get_tweets

""" NLP Stanford CS
import stanza  # https://universaldependencies.org/treebanks/cs_pdt/index.html
stanza.download("cs")  # to se musi nainstalovat predem, ma to 230 Mb
nlp = stanza.Pipeline("cs")
text = "Ukázkový text - věta 1. Tady je věta 2."
doc = nlp(text)
doc_dict_0 = doc.sentences[0].to_dict()
doc_dict_1 = doc.sentences[1].to_dict()
PROPN - lokalita
lemma - zakladni tvar
"""

CURRENCY = {
    "Kč": ["Kč", "CZK", "kč", "KČ"],
    "EUR": ["Euro", "EUR", "€", "eur", "Eur", "EURO"]
}

UNITS = {
    "ks": ["ks", "kusů", "kusy", "kus", "kusech"]
}


def ruian_configuration():
    configuration = ruian.Configuration()
    configuration.api_key['X-API-KEY'] = RUIAN_API
    configuration.host = RUIAN_HOST
    api_instance_public = ruian.PublicApi(ruian.ApiClient(configuration))
    return api_instance_public


"""DATA MINING"""


def get_coordinates(location, city):
    api_instance_public = ruian_configuration()
    api_response = api_instance_public.search_address_ft_api(query=location)
    if city != "":
        if api_response:
            if api_response[0].locality == city and 47.9 <= float(api_response[0].coordinates.wgs.lat) <= 51.2 and \
                    11.9 <= float(api_response[0].coordinates.wgs.lon) <= 19.1:
                lat = api_response[0].coordinates.wgs.lat
                lon = api_response[0].coordinates.wgs.lon
            else:
                time.sleep(0.5)
                nom = Nominatim()
                coord = nom.query(city)
                if coord:
                    lat = coord[0]["lat"]
                    lon = coord[0]["lon"]
                else:
                    lat = None
                    lon = None
        else:
            time.sleep(0.5)
            nom = Nominatim()
            coord = nom.query(city)
            if coord:
                lat = coord[0]["lat"]
                lon = coord[0]["lon"]
            else:
                lat = None
                lon = None
    else:
        lat = None
        lon = None
    coords = [lat, lon]
    return coords


def get_address(lat, lon):
    api_instance_public = ruian_configuration()
    api_response = api_instance_public.nearby_address_wgs_api(lat=lat, lon=lon)
    city = ''
    if api_response:
        city = api_response[0].address.locality
    else:
        time.sleep(0.5)
        nomrev = NominatimReverse()
        adress = nomrev.query(lat=lat, lon=lon)
        if adress:
            adress = dict(adress)
            if 'address' in adress:
                if 'city' in adress['address']:
                    city = adress['address']['city']
                elif 'town' in adress['address']:
                    city = adress['address']['town']
                elif 'village' in adress['address']:
                    city = adress['address']['village']
    return city


LOC_STR_TO_DELETE = r"(česká republika|čr|hlavní město|hl.m.|česká repu|česká republik)"


def clean_address_coords(region="", city="", street=""):
    print('{}(region={}, city={}, street={})'.format('clean_address_coords', region, city, street))
    address = [city, street]  # ruian.search_address neumi kraj
    for item in address:
        index = address.index(item)
        item = item.lower()
        item = re.sub(LOC_STR_TO_DELETE, "", item)
        item = re.sub(" - ", "-", item)
        item = re.sub(",", "", item)
        item = re.sub(" +", " ", item)
        address[index] = item.lstrip().rstrip()
    if street != "":
        address = ", ".join(address)
    else:
        address = address[0]
    coords = get_coordinates(location=address, city=city)
    lat = coords[0]
    lon = coords[1]
    if lat is None or lon is None:
        address = city.split(" ")[0]
        print("to je nova adresa ", address)
        coords = get_coordinates(location=address, city=address)
        print(coords)
        lat = coords[0]
        lon = coords[1]
        city = address  # address_orig[1]
    return lat, lon, city


def elk_clean_and_create_address(pid, region="", city="", street=""):
    if city is None:
        city = ""
    if street is None:
        street = ""
    address_orig = [region, city, street]
    response_post = None
    try:
        locality = ELASTICSEARCH_CLIENT.get(index="krilog-locality", id=pid)["_source"]
    except ApiError:
        response = ELASTICSEARCH_CLIENT.search(
            index="krilog-locality",
            body={
                "query": {
                    "bool": {
                        "must": [
                            {"match": {"region": region}},
                            {"match": {"city": city}},
                            {"match": {"street": street}}
                        ]
                    }
                },
            },
        )
        try:
            results = [result["_source"] for result in response['hits']['hits']]
            locality = results[0]
        except [ValueError, TypeError, KeyError, IndexError]:
            locality = None

    if locality and (locality["coordinates_LAT"] not in [None, ""] and locality["coordinates_LON"] not in [None, ""]):
        locality = locality
    else:
        address = [city, street]  # ruian.search_address neumi kraj
        for item in address:
            index = address.index(item)
            item = item.lower()
            item = re.sub(LOC_STR_TO_DELETE, "", item)
            item = re.sub(" - ", "-", item)
            item = re.sub(",", "", item)
            item = re.sub(" +", " ", item)
            address[index] = item.lstrip().rstrip()
        if street != "":
            address = ", ".join(address)
        else:
            address = address[0]
        coords = get_coordinates(location=address, city=city)
        lat = coords[0]
        lon = coords[1]
        if lat is None or lon is None:
            address = city.split(" ")[0]
            print("to je nova adresa ", address)
            coords = get_coordinates(location=address, city=address)
            print(coords)
            lat = coords[0]
            lon = coords[1]
            address_orig[1] = address

        if locality and (locality["coordinates_LAT"] in [None, ""] or locality["coordinates_LON"] in [None, ""]) and \
                lon is not None and lat is not None:
            response_post = ELASTICSEARCH_CLIENT.update(
                index="krilog-locality",
                id=locality["id"],
                body={"doc": {"coordinates_LAT": lat, "coordinates_LON": lon, "id": locality["id"], "region": region,
                              "city": city, "street": street}}
            )
            print('Proběhla aktualizace lokality s id {}'.format(locality["id"]))

            try:
                time.sleep(0.3)
                locality = ELASTICSEARCH_CLIENT.get(index="krilog-locality", id=locality["id"])["_source"]
            # tohle je snad blbost, to nemuze nastat
            except ApiError:
                response_post = ELASTICSEARCH_CLIENT.index(
                    index="krilog-locality",
                    id=locality["id"],
                    op_type="create",
                    # body=offer,
                    document={"coordinates_LAT": lat, "coordinates_LON": lon, "id": pid, "region": address_orig[0],
                              "city": address_orig[1], "street": address_orig[2]}
                )
                print('Byla vytvorena nova Locality s id {}'.format(locality["id"]))

        elif locality and (lon is None) and (lat is None):
            try:
                locality = ELASTICSEARCH_CLIENT.get(index="krilog-locality", id=locality["id"])["_source"]
            # taky blbost
            except ApiError:
                response_post = ELASTICSEARCH_CLIENT.index(
                    index="krilog-locality",
                    id=locality["id"],
                    op_type="create",
                    document={"coordinates_LAT": lat, "coordinates_LON": lon, "id": pid, "region": address_orig[0],
                              "city": address_orig[1], "street": address_orig[2]}
                )
                print("Byla vytvorena nova Locality s id {}".format(locality["id"]))
        else:
            response_post = ELASTICSEARCH_CLIENT.index(
                index="krilog-locality",
                id=pid,
                op_type="create",
                # body=offer,
                document={"coordinates_LAT": lat, "coordinates_LON": lon, "id": pid, "region": address_orig[0],
                          "city": address_orig[1], "street": address_orig[2]}
            )
            print("Byla vytvorena nova Locality s id {}".format(pid))

            time.sleep(0.3)
            locality = ELASTICSEARCH_CLIENT.get(index="krilog-locality", id=pid)["_source"]
    print('{}: {}'.format('elk_clean_and_create_address', response_post))
    return locality


def re_dict(dict_to_re):
    redict = list(dict.values(dict_to_re))
    redict = list(chain.from_iterable(redict))
    redict = "|".join(redict)
    return redict


def twitter_offer_mining(q, lang, count, word, word_decl, search):
    word_dict = word
    tw_offers, cols, count_results = get_tweets(word=q, lang=lang, count=count, retweet=False)
    offers_dict = []
    if count_results > 0:
        units = re_dict(UNITS)
        curr = re_dict(CURRENCY)

        for offer in tw_offers:
            source = "Twitter"
            twid = offer["id"]
            url = f"https://twitter.com/stanberger_/status/{twid}"
            name_offer = "N/A"  # u Twitteru neni
            name_user = offer["name_user"]
            word = f"{word}"
            search = f"{search}"
            created = time.strftime('%Y-%m-%d %H:%M:%S',
                                    time.strptime(offer['created_at'], '%a %b %d %H:%M:%S +0000 %Y'))
            fulltext = offer["full_text"]
            city = offer["location"]  # casto se v Tw nevyplnuje

            PRICE_RE = r"(\d+[\s\d]*)\s*(" + curr + r")"
            price_all = re.findall(PRICE_RE, fulltext)
            if price_all:
                price = price_all[0][0].replace(" ", "")
                currency = price_all[0][1]
            else:
                price = 0
                currency = "neuvedeno"
            word_re = word.lower()
            if word_decl != "":
                r = r"(\d+[\s\d]*)\s*(" + word_re + r"|" + word_decl + r"|" + units + r")"
            else:
                r = r"(\d+[\s\d]*)\s*(" + word_re + r"|" + units + r")"
            quantity_all = re.findall(r, fulltext.lower())
            if quantity_all:
                quantity = quantity_all[0][0].replace(" ", "")
                units = quantity_all[0][1]
                if units == word_re:
                    units = "ks"
            else:
                quantity = 0
                units = "neuvedeno"

            offer_dict = {
                "id": twid,
                "url": url,
                "name_offer": name_offer,
                "name_user": name_user,
                "word": word_dict,
                "search": search,
                "created": created,
                "fulltext": fulltext,
                "locality": city,
                "price": price,
                "currency": currency,
                "quantity": quantity,
                "units": units,
                "source": source
            }
            offers_dict.append(offer_dict)
            cols = list(offers_dict[0].keys())
    return offers_dict, cols, count_results


def strip_accents(text):
    """
    Strip accents from input String.

    :param text: The input string.
    :type text: String.

    :returns: The processed String.
    :rtype: String.
    """
    try:
        text = str(text, 'utf-8')
    except (TypeError, NameError):  # unicode is a default on python 3
        pass
    text = unicodedata.normalize('NFD', text)
    text = text.encode('ascii', 'ignore')
    text = text.decode("utf-8")
    return str(text)


def quantity_mining(word, name, fulltext, word_decl, word_decl_re):
    units_all = re_dict(UNITS)
    # units_found = ""

    word_re = word.lower()
    # word = f"{word}"
    if word_decl_re != "":
        r = r"(\d+[\s\d]*)\s*(" + word_re + r"|" + word_decl + r"|" + units_all + r")"
    else:
        r = r"(\d+[\s\d]*)\s*(" + word_re + r"|" + units_all + r")"
    quantity_all = re.findall(r, name.lower())
    if quantity_all:
        quantity_source = quantity_all[0][0]
        quantity_list = quantity_source.split(
            " ")  # s tim jeste neco vymyslet - jen kdyz mezera oddeluje trojice od konce
        try:
            quantity = int(quantity_list[-1])
        except [ValueError, TypeError, KeyError, IndexError]:
            try:
                quantity = int(quantity_list[-2])
            except [ValueError, TypeError, KeyError, IndexError]:
                quantity = 0
        if quantity > 0:
            units_found = quantity_all[0][1]
            if units_found in [word_re, word_decl]:
                units_found = "ks"
            elif units_found in units_all:
                for k, v in UNITS.items():
                    if units_found in v:
                        units_found = k
                        break
            else:
                units_found = ""
        else:
            units_found = "neuvedeno"

    else:
        quantity_all = re.findall(r, fulltext.lower())
        if quantity_all:
            quantity_source = quantity_all[0][0]
            quantity_list = quantity_source.split(
                " ")  # s tim jeste neco vymyslet - jen kdyz mezera oddeluje trojice od konce
            try:
                quantity = int(quantity_list[-1])
            except [ValueError, TypeError, KeyError, IndexError]:
                try:
                    quantity = int(quantity_list[-2])
                except [ValueError, TypeError, KeyError, IndexError]:
                    quantity = 0
            if quantity > 0:
                units_found = quantity_all[0][1]
                if units_found in [word_re, word_decl]:
                    units_found = "ks"
                elif units_found in units_all:
                    for k, v in UNITS.items():
                        if units_found in v:
                            units_found = k
                            break
                else:
                    units_found = ""
            else:
                units_found = "neuvedeno"

        else:
            quantity = 0  # [0]
            units_found = "neuvedeno"  # [0][1]

    return {"quantity": quantity, "units": units_found}


def price_mining(name, fulltext):
    curr = re_dict(CURRENCY)
    PRICE_RE = r"(\d+[\s\d]*)\s*,?\s*-?\s*(" + curr + r")"

    price_all_name = re.findall(PRICE_RE, name)
    if price_all_name:
        price_source = price_all_name[0][0]
        price_list = price_source.split(" ")
        try:
            price = int(price_list[-1])
        except [ValueError, TypeError, KeyError, IndexError]:
            try:
                price = int(price_list[-2])
            except [ValueError, TypeError, KeyError, IndexError]:
                price = 0
        currency_found = price_all_name[0][1]
    else:
        price_all_fulltext = re.findall(PRICE_RE, fulltext)
        if price_all_fulltext:
            price_source = price_all_fulltext[0][0]
            price_list = price_source.split(" ")
            try:
                price = int(price_list[-1])
            except [ValueError, TypeError, KeyError, IndexError]:
                try:
                    price = int(price_list[-2])
                except [ValueError, TypeError, KeyError, IndexError]:
                    price = 0
            currency_found = price_all_fulltext[0][1]
        else:
            price = 0
            currency_found = 'neuvedeno'
    for k, v in CURRENCY.items():
        if currency_found in v:
            currency_found = k
            break

    return {'price': price, 'currency': currency_found}



# from data_mining.preprocess import strip_accents, price_mining, quantity_mining, elk_clean_and_create_address
import json
from datetime import datetime
from bs4 import BeautifulSoup
import requests
import re
from math import ceil

from elasticsearch import ApiError

from shopping.elk import ELASTICSEARCH_CLIENT, filter_by_word_and_source_elk
# from data_mining.elk import ELASTICSEARCH_CLIENT, filter_by_word_and_source_elk

from shopping.preprocess import strip_accents, price_mining, quantity_mining, elk_clean_and_create_address


def sbazar_one_word(exp, blacklist, wordcore, word_decl, word_decl_re):
    offers = []
    expression = strip_accents(exp.lower()).replace(
        " ", "%20")  # převedení výrazu do podoby přímo zadatelné do sbazarové URL. Skloňuje sám, synonyma netřeba

    # počítadlo stránek
    page_number = 1
    next_number = True

    while next_number:
        response = requests.get(
            "https://www.sbazar.cz/hledej/{}/0-vsechny-kategorie/cela-cr/cena-neomezena/nejnovejsi/{}".format(
                expression, str(page_number)))
        soup = BeautifulSoup(response.text, "lxml")
        # zkoumám, zda existuje další stránka inzerátů - pokud ano, zvednu číslo stránky, jinak zastavuji cyklus
        list_of_links = soup.find_all('a', href=True)
        for tag in list_of_links:
            if tag.text == "Další":
                page_number += 1
                next_number = True
                break
            else:
                next_number = False

        # dolování seznamu inzerátů v json
        script_source = soup.find("script", src=None)
        pattern = "IMA.Cache = .+?;\n"
        raw_data = re.findall(pattern, script_source.string, re.S)
        raw_data2 = raw_data[0].split('"results":')
        b = None
        for a in raw_data2:
            if "images" in a:
                b = a
                break
        # nalezen správný kus kódu začínající results: a obsahující inzeráty - konec je potřeba nalézt počítáním závorek a pak převést do json
        if b:
            count_of_starts = 0
            count_of_ends = 0
            results_final = ""

            for znak in b:
                if znak == "[":
                    count_of_starts = count_of_starts + 1
                elif znak == "]":
                    count_of_ends = count_of_ends + 1

                results_final += znak

                if count_of_starts == count_of_ends and count_of_starts > 0:
                    break

            output = json.loads(results_final)
            # seznam inzerátů hotov, kromě description a dopočítávaných věcí už obsahují všechny údaje
            for x in output:
                my_string = None
                # vyřazuji inzeráty obsahující v nadpisu něco s blacklistových slov
                if any(word in x["name"].lower() for word in blacklist):
                    pass
                # vyřazuji inzeráty NEobsahující v nadpisu definovaný kořen slova
                elif any(core in x["name"].lower() for core in wordcore):
                    # description = "neuvedeno"  # predefine kvůli vytěžování - pokud existuje, přepíše se
                    username = ""
                    # ošetřeny nalezené výjimky - jiné eshopy apod., může tam být něco dalšího
                    if "user" in x and x["user"] is not None:
                        x["link"] = "https://www.sbazar.cz/" + x["user"]["user_service"]["shop_url"] + "/detail/" + x[
                            "seo_name"]
                        username = x["user"]["user_service"]["shop_url"]
                    elif "shop" in x:
                        username = x["shop"]["name"]
                        x["link"] = "https://www.sbazar.cz/rozbalena-nabidka/" + x["seo_name"] + "-" + x["id"]
                    elif "premise" in x:
                        username = x["premise"]["name"]
                        x["link"] = "https://www.sbazar.cz/" + str(x["premise"]["id"]) + "/detail/" + x["seo_name"]
                    else:
                        x["link"] = "https://www.sbazar.cz/"

                    # pro každý inzerát je potřeba ještě extra otevřít detail kvůli description, doluje se podobně
                    response_item = requests.get(x["link"])
                    soup_item = BeautifulSoup(response_item.text, "lxml")
                    script_source_item = soup_item.find("script", src=None)
                    pattern = "IMA.Cache = .+?;\n"
                    raw_data_item = re.findall(pattern, script_source_item.string, re.S)
                    raw_data_3 = raw_data_item[0].split('"result":')

                    for item_anno in raw_data_3:
                        if "description" in item_anno:
                            my_string = item_anno

                    # nalezen správný json s údaji o inzerátu
                    if my_string:
                        count_of_starts = 0
                        count_of_ends = 0
                        results_final = ""

                        for letter in my_string:
                            if letter == "{":
                                count_of_starts = count_of_starts + 1
                            elif letter == "}":
                                count_of_ends = count_of_ends + 1

                            results_final += letter

                            if count_of_starts == count_of_ends and count_of_starts > 0:
                                break

                        output2 = json.loads(results_final)
                        description = output2["description"] if output2["description"] else "Neuvedeno"
                        # vyřazuji inzeráty obsahující v popisu něco s blacklistových slov - pokud není, pokračuji zbylými údaji
                        if any(word in description.lower() for word in blacklist):
                            pass
                        else:
                            price = x["price"]
                            currency_offer = "Kč"
                            if price == 0:
                                price_preproc = price_mining(x["name"], description)
                                price = price_preproc["price"]
                                currency_offer = price_preproc["currency"]

                            # některé nabídky nemají lokaci, typicky jiné eshopy, které tam něco přeprodávají
                            if "locality" in x and "region" in x["locality"]:
                                locality = {
                                    "pid": "0" + str(x["id"]),
                                    # pridavam 0 pred id - bazos ma taky 9mistna id a pomlatilo by se to - tam pridavam 9
                                    "region": x["locality"]["region"],
                                    "street": x["locality"]["street"]
                                }
                                if x["locality"]["entity_type"] == "district":
                                    locality["city"] = x["locality"]["district"]
                                elif x["locality"]["entity_type"] == "quarter":
                                    locality["city"] = x["locality"]["quarter"]
                                else:  # pro entity_type municipality, ward a prip. neco dalsiho
                                    locality["city"] = x["locality"]["municipality"]
                            else:
                                locality = {
                                    "pid": "0" + str(x["id"]),
                                    "region": "",
                                    "city": "",
                                    "street": ""
                                }
                            offer_dict = {
                                "id": x["id"],
                                "url": x["link"],
                                "name_offer": x["name"],
                                "name_user": username,
                                "created": x["create_date"].replace("T", " ") if "create_date" in x else datetime.now(),
                                "fulltext": description,
                                "locality": locality,
                                "price": price,
                                "currency": currency_offer,
                                "quantity": quantity_mining(
                                    word=exp, name=x["name"], fulltext=description,
                                    word_decl=word_decl, word_decl_re=word_decl_re)["quantity"],
                                "units": quantity_mining(
                                    word=exp, name=x["name"], fulltext=description,
                                    word_decl=word_decl, word_decl_re=word_decl_re)["units"],
                                "source": "Sbazar"
                            }
                            offers.append(offer_dict)
                else:
                    pass
    return offers


def bazos_one_word(exp, blacklist, wordcore, word_decl, word_decl_re):
    offers = []
    expression = strip_accents(exp.lower()).replace(
        " ",
        "+")  # převedení výrazu do podoby přímo zadatelné do bazošové URL. Skloňuje sám, synonyma netřeba

    # zjistim pocet stranek pro dany vyraz
    response = requests.get(
        "https://www.bazos.cz/search.php?hledat={}&rubriky=www&hlokalita=&humkreis=25&cenaod=&cenado=&Submit=Hledat&kitx=ano".format(
            expression))
    soup = BeautifulSoup(response.text, "lxml")
    anno_count_pool = soup.find('div', {'class', 'listainzerat inzeratyflex'})
    count_text = anno_count_pool.find('div', {'class', 'inzeratynadpis'}).text
    count = int(count_text.split("inzerátů z ", 1)[1].replace(" ", ""))
    no_of_pages = ceil(count / 20)

    # počítadlo stránek
    page_number = 0

    while page_number < no_of_pages:
        # TODO osetrit error 403
        response = requests.get(
            "https://www.bazos.cz/search.php?hledat={}&hlokalita=&humkreis=25&cenaod=&cenado=&order=&crz={}".format(
                expression, str(page_number * 20)))
        soup = BeautifulSoup(response.text, "lxml")
        anno_list = soup.find_all('div', {'class', 'inzeraty inzeratyflex'})

        for anno in anno_list:
            name_offer = anno.find('h2', {'class', 'nadpis'}).text
            # vyřazuji inzeráty obsahující v nadpisu něco s blacklistových slov
            name_user = None
            if any(word in name_offer.lower() for word in blacklist):
                pass
            # vyřazuji inzeráty NEobsahující v nadpisu definovaný kořen slova
            elif any(core in name_offer.lower() for core in wordcore):
                fulltext = anno.find('div', {'class', 'popis'}).text
                # vyřazení inzerátů obsahující ve fulltextu něco s blacklistových slov
                if any(word in fulltext.lower() for word in blacklist):
                    pass
                else:
                    pricepool = anno.find('div', {'class', 'inzeratycena'}).text
                    try:
                        price = int(re.search('(.+?) Kč', str(pricepool)).group(1).replace(" ", ""))
                    except [TypeError, ValueError]:
                        price = 0
                    currency = "Kč"
                    if price == 0:
                        price_preproc = price_mining(name_offer, fulltext)
                        price = price_preproc["price"]
                        currency = price_preproc["currency"]

                    '''
                    city_plus_psc = anno.find('div', {'class','inzeratylok'}).text
                    city = re.split(r'\d\d\d \d\d', city_plus_psc)[0]
                    '''

                    url = anno.a["href"]
                    # idpool = re.search('\/inzerat(.+?)\/', str(url))
                    idpool = re.search('/inzerat(.+?)/', str(url))
                    identifier = idpool.group(
                        1)  # id pro bazoš začíná lomítkem - možno odlišit příp. jinak, ale je to nutné - má taky 9místná čísla jako Sbazar, může se pomlátit, už se mi to v jednom případě stalo
                    datepool = anno.find("span", {'class', 'velikost10'})
                    # datepool2 = re.search('\[(.+?)\]', str(datepool))
                    datepool2 = re.search('[(.+?)]', str(datepool))
                    if datepool2:
                        date = datepool2.group(1)
                    else:
                        date = None

                    # pro každý inzerát je potřeba ještě extra otevřít detail kvůli jménu usera a souřadnicím
                    response_detail = requests.get(url)
                    soup_detail = BeautifulSoup(response_detail.text, "lxml")
                    detail_pool = soup_detail.find_all('td')
                    namepool = ""
                    # name = ""
                    # lat = None
                    # lon = None
                    for cell in detail_pool:
                        if "Jméno:" in str(cell):
                            namepool = cell
                            break

                    a_list = namepool.find_all("a")
                    map_link = None
                    for x in a_list:
                        if "jmeno" in str(x):
                            name_user = x.text
                        if "google.com/maps" in str(x):
                            map_link = x["href"]

                    # coordinates_str = re.search('google.com\/maps\/place\/(.*)\/@', map_link)
                    coordinates_str = re.search('google.com/maps/place/(.*)/@', map_link)
                    c = coordinates_str.group(1)
                    lat = float(c.split(",")[0])
                    lon = float(c.split(",")[1])

                    # detail inzerátu už obsahuje souřadnice, podle nich se hledá město
                    locality = {
                        "pid": "8" + str(identifier),  # 8 náhodně vybrána pro jednoznačnou identifikaci
                        "lat": lat,
                        "lon": lon
                    }
                    offer_dict = {
                        "id": identifier,
                        "url": url,
                        "name_offer": name_offer,
                        "name_user": name_user,
                        "created": datetime.strptime(date, "%d.%m. %Y") if date else datetime.now(),
                        "fulltext": fulltext,
                        "locality": locality,
                        "price": price,
                        "currency": currency,
                        "quantity": quantity_mining(word=exp, name=name_offer, fulltext=fulltext,
                                                    word_decl=word_decl, word_decl_re=word_decl_re)["quantity"],
                        "units": quantity_mining(word=exp, name=name_offer, fulltext=fulltext,
                                                 word_decl=word_decl, word_decl_re=word_decl_re)["units"],
                        "source": "Bazoš"
                    }
                    offers.append(offer_dict)
            else:
                pass
        page_number = page_number + 1
    return offers


def elk_sbazar_one_word(word_id, word_decl, word_decl_re, delete_existing=True):
    word_object = ELASTICSEARCH_CLIENT.get(index="krilog-word", id=word_id)["_source"]
    exp = word_object["name"]
    blacklist = [res["name"] for res in word_object["restricted_words"]]
    wordcore = [word_object["word_core"], strip_accents(word_object["word_core"].lower())]

    if delete_existing:
        offer_actual = filter_by_word_and_source_elk(word_id, "Sbazar")
        for offer_old in offer_actual:
            ELASTICSEARCH_CLIENT.delete(index="krilog-offer", id=offer_old["id"])
            print("Offer s id {} byl vymazán z elk".format(offer_old["id"]))

    expression = strip_accents(exp.lower()).replace(" ",
                                                    "%20")  # převedení výrazu do podoby přímo zadatelné do sbazarové URL. Skloňuje sám, synonyma netřeba

    # počítadlo stránek
    page_number = 1
    next_number = True

    while next_number:
        response = requests.get(
            "https://www.sbazar.cz/hledej/{}/0-vsechny-kategorie/cela-cr/cena-neomezena/nejnovejsi/{}".format(
                expression, str(page_number)))
        soup = BeautifulSoup(response.text, "lxml")
        # zkoumám, zda existuje další stránka inzerátů - pokud ano, zvednu číslo stránky, jinak zastavuji cyklus
        list_of_links = soup.find_all('a', href=True)
        for tag in list_of_links:
            if tag.text == "Další":
                page_number += 1
                next_number = True
                break
            else:
                next_number = False

        # dolování seznamu inzerátů v json
        script_source = soup.find("script", src=None)
        pattern = "IMA.Cache = .+?;\n"
        raw_data = re.findall(pattern, script_source.string, re.S)
        raw_data2 = raw_data[0].split('"results":')
        b = None
        for a in raw_data2:
            if "images" in a:
                b = a
                break
        # nalezen správný kus kódu začínající results: a obsahující inzeráty - konec je potřeba nalézt počítáním závorek a pak převést do json
        if b:
            count_of_starts = 0
            count_of_ends = 0
            results_final = ""

            for znak in b:
                if znak == "[":
                    count_of_starts = count_of_starts + 1
                elif znak == "]":
                    count_of_ends = count_of_ends + 1

                results_final += znak

                if count_of_starts == count_of_ends and count_of_starts > 0:
                    break

            output = json.loads(results_final)
            # seznam inzerátů hotov, kromě description a dopočítávaných věcí už obsahují všechny údaje
            for x in output:
                # vyřazuji inzeráty obsahující v nadpisu něco s blacklistových slov
                if any(word in x["name"].lower() for word in blacklist):
                    pass
                # vyřazuji inzeráty NEobsahující v nadpisu definovaný kořen slova
                elif any(core in x["name"].lower() for core in wordcore):
                    # description = "neuvedeno"  # predefine kvůli vytěžování - pokud existuje, přepíše se
                    username = ""
                    # ošetřeny nalezené výjimky - jiné eshopy apod., může tam být něco dalšího
                    if "user" in x and x["user"] is not None:
                        x["link"] = "https://www.sbazar.cz/" + x["user"]["user_service"]["shop_url"] + "/detail/" + x[
                            "seo_name"]
                        username = x["user"]["user_service"]["shop_url"]
                    elif "shop" in x:
                        username = x["shop"]["name"]
                        x["link"] = "https://www.sbazar.cz/rozbalena-nabidka/" + x["seo_name"] + "-" + x["id"]
                    elif "premise" in x:
                        username = x["premise"]["name"]
                        x["link"] = "https://www.sbazar.cz/" + str(x["premise"]["id"]) + "/detail/" + x["seo_name"]
                    else:
                        x["link"] = "https://www.sbazar.cz/"

                    # pro každý inzerát je potřeba ještě extra otevřít detail kvůli description, doluje se podobně
                    response_item = requests.get(x["link"])
                    soup_item = BeautifulSoup(response_item.text, "lxml")
                    script_source_item = soup_item.find("script", src=None)
                    pattern = "IMA.Cache = .+?;\n"
                    raw_data_item = re.findall(pattern, script_source_item.string, re.S)
                    raw_data_3 = raw_data_item[0].split('"result":')
                    my_string = None
                    for item_anno in raw_data_3:
                        if "description" in item_anno:
                            my_string = item_anno

                    # nalezen správný json s údaji o inzerátu
                    if my_string:
                        count_of_starts = 0
                        count_of_ends = 0
                        results_final = ""

                        for letter in my_string:
                            if letter == "{":
                                count_of_starts = count_of_starts + 1
                            elif letter == "}":
                                count_of_ends = count_of_ends + 1

                            results_final += letter

                            if count_of_starts == count_of_ends and count_of_starts > 0:
                                break

                        output2 = json.loads(results_final)
                        description = output2["description"] if output2["description"] else "Neuvedeno"
                        # vyřazuji inzeráty obsahující v popisu něco s blacklistových slov - pokud není, pokračuji zbylými údaji
                        if any(word in description.lower() for word in blacklist):
                            pass
                        else:
                            price = x["price"]
                            currency_offer = "Kč"
                            if price == 0:
                                price_preproc = price_mining(x["name"], description)
                                price = price_preproc["price"]
                                currency_offer = price_preproc["currency"]

                            # některé nabídky nemají lokaci, typicky jiné eshopy, které tam něco přeprodávají
                            if "locality" in x and "region" in x["locality"]:
                                if x["locality"]["entity_type"] == "district":
                                    locality = elk_clean_and_create_address(
                                        pid="0" + str(x["id"]),
                                        # pridavam 0 pred id - bazos ma taky 9mistna id a pomlatilo by se to - tam pridavam 9
                                        region=x["locality"]["region"],
                                        city=x["locality"]["district"],
                                        street=x["locality"]["street"])
                                elif x["locality"]["entity_type"] == "quarter":
                                    locality = elk_clean_and_create_address(
                                        pid="0" + str(x["id"]),
                                        # pridavam 0 pred id - bazos ma taky 9mistna id a pomlatilo by se to - tam pridavam 9
                                        region=x["locality"]["region"],
                                        city=x["locality"]["quarter"],
                                        street=x["locality"]["street"])
                                else:  # pro entity_type municipality, ward a prip. neco dalsiho
                                    locality = elk_clean_and_create_address(
                                        pid="0" + str(x["id"]),
                                        # pridavam 0 pred id - bazos ma taky 9mistna id a pomlatilo by se to - tam pridavam 9
                                        region=x["locality"]["region"],
                                        city=x["locality"]["municipality"],
                                        street=x["locality"]["street"])

                            else:
                                locality = elk_clean_and_create_address(
                                    pid="0" + str(x["id"]),
                                    region="",
                                    city="",
                                    street="")

                            # uložení nabídky
                            # podmínku mazání ponechat, i když je nahoře delete všeho - jeden inzerát se může najít vícekrát pro různé výrazy a docházelo by k jeho multiplikaci
                            try:
                                # offer_exists = es.get(index="krilog-offer", id=x["id"])["_source"]
                                ELASTICSEARCH_CLIENT.delete(index="krilog-offer", id=x["id"])
                                print("Offer s id {} byl vymazán z elk".format(x["id"]))
                            except ApiError:
                                pass

                            anno = {"id": x["id"],
                                    "url": x["link"],
                                    "name_offer": x["name"],
                                    "name_user": username,
                                    "word": {"id": word_id, "name": exp},
                                    "created": x[
                                                   "create_date"] + ".000Z" if "create_date" in x else "2000-01-01T12:00:00.000Z",
                                    "price": price,
                                    "currency": currency_offer,
                                    "quantity": quantity_mining(word=exp, name=x["name"], fulltext=description,
                                                                word_decl=word_decl, word_decl_re=word_decl_re)[
                                        "quantity"],
                                    "units": quantity_mining(word=exp, name=x["name"], fulltext=description,
                                                             word_decl=word_decl, word_decl_re=word_decl_re)["units"],
                                    "fulltext": description,
                                    "locality": locality,
                                    "source": "Sbazar"
                                    }

                            try:
                                ELASTICSEARCH_CLIENT.update(
                                    index="krilog-offer",
                                    id=x["id"],
                                    body={"doc": anno}
                                )
                                print("Proběhla aktualizace Offer s id {}".format(x["id"]))

                            except ApiError:
                                ELASTICSEARCH_CLIENT.index(
                                    index="krilog-offer",
                                    id=x["id"],
                                    op_type="create",
                                    # body=offer,
                                    document=anno
                                )
                                print("Byla vytvorena nova Offer s id {}".format(x["id"]))

                else:
                    pass

    response = ELASTICSEARCH_CLIENT.search(
        index="krilog-offer",
        body={
            "query": {
                "bool": {
                    "must": [
                        {"match": {"word.id": word_id}},
                        {"match": {"source": "Sbazar"}}
                    ]
                }
            },
        },
    )

    offers_new = [result["_source"] for result in response['hits']['hits']]

    return offers_new



import tweepy

from shopping.config import TWITTER_CONSUMER_KEY, TWITTER_CONSUMER_SECRET, TWITTER_ACCESS_TOKEN, \
    TWITTER_ACCESS_TOKEN_SECRET, TWITTER_BEARER_TOKEN

twitter_auth_key = {
    "consumer_key": TWITTER_CONSUMER_KEY,
    "consumer_secret": TWITTER_CONSUMER_SECRET,
    "access_token": TWITTER_ACCESS_TOKEN,
    "access_token_secret": TWITTER_ACCESS_TOKEN_SECRET,
    "bearer_token": TWITTER_BEARER_TOKEN
}


def get_tweets(word, lang, count, retweet):
    auth = tweepy.OAuthHandler(
        twitter_auth_key["consumer_key"],
        twitter_auth_key["consumer_secret"]
    )
    auth.set_access_token(
        twitter_auth_key["access_token"],
        twitter_auth_key["access_token_secret"]
    )
    API = tweepy.API(auth)
    results = [status._json for status in
               tweepy.Cursor(API.search_tweets, q=word, lang=lang, tweet_mode='extended').items(count)]
    my_tweets = []
    cols = []
    count_results = len(results)
    for result in results:
        keys = ("created_at", "full_text", "id", "user")
        res = {k: result[k] for k in keys}
        res["name_user"] = res["user"]["name"]
        res["user"] = res["user"]["location"]
        res["location"] = res.pop("user")
        # twid = res["id"]
        if retweet:
            if 'retweeted_status' in result.keys():
                retweet = result["retweeted_status"]
                retweet = {k: retweet[k] for k in keys}
                retweet["created_at_orig"] = retweet.pop("created_at")
                retweet["full_text_orig"] = retweet.pop("full_text")
                retweet["id_orig"] = retweet.pop("id")
            else:
                retweet = {
                    "created_at_orig": "",
                    "full_text_orig": "",
                    "id_orig": ""
                }
            res = {**res, **retweet}
        my_tweets.append(res)
        cols = list(res.keys())
    return my_tweets, cols, count_results


# https://www.programcreek.com/python/example/76301/tweepy.Cursor


def get_tweets_client(word, count):
    auth = twitter_auth_key["bearer_token"]
    api = tweepy.Client(auth)

    response = api.search_recent_tweets(
        query=word,
        max_results=count,
        tweet_fields=['text'],
        # tweet_fields=['author_id', 'created_at', 'text', 'source', 'lang', 'geo'],
        # user_fields=['name', 'username', 'location', 'verified'],
        # expansions=['geo.place_id', 'author_id'],
        place_fields=['country', 'country_code'])
    search_results = response.data
    return search_results



import os

ELK_URL = os.getenv('ELK_URL', 'elk_address')
ELK_USERNAME = os.getenv('ELK_USERNAME', 'elk_username')
ELK_PASSWORD = os.getenv('ELK_PASSWORD', 'elk_passwd')
ELK_CERTS_DIR = os.getenv('ELK_CERTS_DIR', '')
ELK_CERTS_CA = ''
if ELK_CERTS_DIR != '':
    ELK_CERTS_CA = os.path.join(ELK_CERTS_DIR, 'ca', 'ca.pem')
ELK_VERIFY_CERTS = os.getenv("ELK_VERIFY_CERTS", 'False').lower() in ('true', '1', 't')

TWITTER_CONSUMER_KEY = os.getenv('TWITTER_CONSUMER_KEY', 'abc')
TWITTER_CONSUMER_SECRET = os.getenv('TWITTER_CONSUMER_SECRET', 'def')
TWITTER_ACCESS_TOKEN = os.getenv('TWITTER_ACCESS_TOKEN', 'ghi')
TWITTER_ACCESS_TOKEN_SECRET = os.getenv('TWITTER_ACCESS_TOKEN_SECRET', 'jkl')
TWITTER_BEARER_TOKEN = os.getenv('TWITTER_BEARER_TOKEN', 'brum')

RUIAN_API = os.getenv("RUIAN_API", "aaa")
RUIAN_HOST = os.getenv("RUIAN_HOST", "bbb")


class ShopingError(Exception):
    pass




from elasticsearch import Elasticsearch, ApiError

from shopping.config import ELK_URL, ELK_USERNAME, ELK_PASSWORD, ELK_CERTS_CA, ELK_VERIFY_CERTS


# import config


def create_es_client(
        url=ELK_URL,
        user=ELK_USERNAME,
        password=ELK_PASSWORD,
        ca_certs=ELK_CERTS_CA,
        verify_certs=ELK_VERIFY_CERTS):
    try:
        if url is None:
            return None
        if ca_certs not in ['', None]:
            if verify_certs:
                out = Elasticsearch(
                    hosts=str(url),
                    ca_certs=str(ca_certs),
                    http_auth=(user, password),
                    verify_certs=True)
            else:
                out = Elasticsearch(
                    hosts=str(url),
                    http_auth=(user, password),
                    verify_certs=False)
        else:
            out = Elasticsearch(
                hosts=str(url),
                http_auth=(user, password),
                verify_certs=False)
    # except elasticsearch.ElasticsearchException:
    except ApiError:
        out = None
    return out


ELASTICSEARCH_CLIENT = create_es_client()


def filter_by_word_elk(index, word):
    response = ELASTICSEARCH_CLIENT.search(
        index=index,
        body={
            "query": {
                "match": {"word.id": word}
            },
            "size": 10000
        },
    )

    results = [result["_source"] for result in response['hits']['hits']]
    count = response['hits']['total']['value']

    return {"results": results, "count": count}


def get_all_elk(index):
    response = ELASTICSEARCH_CLIENT.search(
        index=index,
        body={
            "query": {
                "match_all": {}
            },
            "size": 10000
        }
    )

    results = [result["_source"] for result in response['hits']['hits']]
    count = response['hits']['total']['value']

    return {"results": results, "count": count}


def filter_by_word_and_source_elk(word, source):
    response = ELASTICSEARCH_CLIENT.search(
        index="krilog-offer",
        body={
            "query": {
                "bool": {
                    "must": [
                        {"match": {"word.id": word}},
                        {"match": {"source": source}}
                    ]
                }
            },
            "size": 10000
        },
    )

    results = [result["_source"] for result in response['hits']['hits']]
    return results


def filter_by_source_elk(source):
    response = ELASTICSEARCH_CLIENT.search(
        index="krilog-offer",
        body={
            "query": {
                "match": {"source": source}
            },
            "size": 10000
        },
    )

    results = [result["_source"] for result in response['hits']['hits']]

    return results


def get_words():
    try:
        response = ELASTICSEARCH_CLIENT.search(
            index="krilog-word",
            body={
                "query": {
                    "match_all": {}
                },
                "size": 100
            },
        )
        values = [[item["_source"]["id"], item["_source"]["name"]] for item in response['hits']['hits']]
        return values
    except ApiError:
        return [["", ""]]


def get_demands():
    try:
        response = ELASTICSEARCH_CLIENT.search(
            index="krilog-demand",
            body={
                "query": {
                    "match_all": {}
                },
                "size": 10000
            },
        )
        values = [[item["_source"]["id"], "{}, {} {}, {}".format(
            item["_source"]["word"]["name"], item["_source"]["quantity"],
            item["_source"]["units"], item["_source"]["organization"])] for item in response['hits']['hits']]
        return values
    except ApiError:
        return [["", ""]]
